#include "stdafx.h"
#include "math.h"
#include "clf_geometry.h"

// check if the two line segments (p1, p2) and (p3,p4) intersect
bool intersect(const clf_point2D& p1, const clf_point2D& p2, const clf_point2D& p3, const clf_point2D& p4, clf_point2D* p0)
{
	// first check horizontal and vertical gap
	if (min(p1.x, p2.x) > max(p3.x,p4.x))
		return false;
	if (max(p1.x, p2.x) < min(p3.x,p4.x))
		return false;
	if (min(p1.y, p2.y) > max(p3.y,p4.y))
		return false;
	if (max(p1.y, p2.y) < min(p3.y,p4.y))
		return false;

	// firsty calculate the common denominator
	float d = (p4.y-p3.y)*(p2.x-p1.x)-(p4.x-p3.x)*(p2.y-p1.y);
	if (fabs(d) < 0.0000001) // almost 0 (compensate for computer accuracy)
		return false;

	// calculate fractional part for first segment
	float na = ((p4.x-p3.x)*(p1.y-p3.y)-(p4.y-p3.y)*(p1.x-p3.x))/d;
	if (na < 0.f || na > 1.f) // beyond limits of first segment, search no more
		return false;

	// calculate fractional part for second segment
	float nb = ((p2.x-p1.x)*(p1.y-p3.y)-(p2.y-p1.y)*(p1.x-p3.x))/d;
	if (nb < 0.f || nb > 1.f) // beyond limits of second segment, search no more
		return false;

	// we have intersection, calculate actual coordinates (if required)
	if (p0)
	{
		p0->x = p1.x+na*(p2.x-p1.x);
		p0->y = p1.y+na*(p2.y-p1.y);
		// or using the second segment endpoints
		// p0->x = p3.x+nb*(p4.x-p3.x);
		// p0->y = p3.y+nb*(p4.y-p3.y);

	}

	// return true
	return true;
}

// check if the point (pt) is internal to the polygon (poly)
bool point_in_polygon(clf_polygon2D& poly, clf_point2D pt)
{
	// first check if the point is out of the bounds of the polygon
	if (pt.x < poly.bounds.left || pt.x > poly.bounds.right || pt.y < poly.bounds.bottom || pt.y > poly.bounds.top)
		return false;

	// generate a point out of the polygon with the same y coordinate as the check point
	clf_point2D right(poly.bounds.right+1.f, pt.y);
	clf_point2D isect;

	// now iterate through the points of the polygon
	clf_polygon2D::iterator next = poly.begin();
	clf_polygon2D::iterator it;
	int count = 0;
	for (it=poly.begin() ; it!=poly.end(); ++it)
	{
		// -next- points to the 'next' point while -it- is the current
		++next;
		if (next == poly.end())
			next = poly.begin();

		if (intersect(*it, *next, pt, right, &isect))
		{
			if (fabs(it->y-isect.y) < 0.001) // the benefit of horizontal lines is that we only need to check the y coordinate
			{
				// ignore it, we will catch it as ending point later
			}
			// if the ending point of the segment is the intersection
			else if (fabs(next->y-isect.y) < 0.001) // the benefit of horizontal lines is that we only need to check the y coordinate
			{
				clf_polygon2D::iterator next_next = next;
				++next_next;
				if (next_next == poly.end())
					next_next = poly.begin();
				// check if the two segments are on the same side of the horizontal line
				if ((next_next->y-next->y)*(next->y-it->y) > 0) // dy1*dy2>0 ==> the polygon intersects
				{
					count++; // boundary crossing, count the intersection
				}
				else
				{
					// not an actual intersection, do not count it
				}
			}
			else
			{ // intersection point, count it
				count ++;
			}
		}
	}

	if (count%2) // odd number of intersections
		return true;

	return false;
}

// check if the two polygons collide
bool polygons_collide(clf_polygon2D& poly1, clf_polygon2D& poly2)
{
	// first check if they are valid
	if (poly1.size() < 2 || poly2.size() < 2)
		return false;
	// then if they are near enough
	if (poly2.bounds.right < poly1.bounds.left || poly2.bounds.left > poly1.bounds.right || 
			poly2.bounds.top < poly1.bounds.bottom || poly2.bounds.bottom > poly1.bounds.top)
		return false;

	// now if there is an intersection between their edges
	clf_polygon2D::iterator it1;
	clf_polygon2D::iterator it2;
	clf_polygon2D::iterator next1=poly1.begin();
	++next1;
	for (it1=poly1.begin(); it1!=poly1.end(); ++it1)
	{
		if (next1 == poly1.end()) next1 = poly1.begin();
		clf_polygon2D::iterator next2=poly2.begin();
		++next2;
		for (it2=poly2.begin(); it2!=poly2.end(); ++it2)
		{
			if (next2 == poly2.end()) next2 = poly2.begin();
			if (intersect(*it1, *next1, *it2, *next2, NULL))
				return true;
			++next2;
		}
		++next1;
	}

	// final check, if we want to cover all bases check weather one polygon is inside the other
	it1=poly1.begin();
	if (point_in_polygon(poly2, *it1)) // one point is enough
		return true;
	it2=poly2.begin();
	if (point_in_polygon(poly1, *it2))
		return true;

	return false;
}

// rotate point (x,y) around center (cx,cy) by (angle)
void rotate2D(float cx, float cy, float *x, float *y, float angle)
{
	float dx = *x - cx;
	float dy = *y - cy;

	*x = cx + dx*cos(angle) - dy*sin(angle);
	*y = cy + dx*sin(angle) + dy*cos(angle);
}
